import re


class CallerAttributesComposer:

    def __init__(self) -> None:
        pass

    def compose(self, *, id_: str = 'MoUpdateOrchestrator', interactive: int = 1, shedding_aware: int = 1) -> str:
        caller_attributes = {
            'Id': id_,
            'Interactive': interactive,
            'SheddingAware': shedding_aware,
        }

        return 'E:' + '&'.join(f'{k}={v}' for k, v in caller_attributes.items())


class DeviceAttributesComposer:

    def __init__(self) -> None:
        pass

    def _get_branch_from_build(self, build: str) -> str:
        build = re.match(r'^(\d+)\.(\d+)\.(\d+)\.(\d+)$', build).group(3)

        match build:
            case '15063':
                return 'rs2_release'
            case '16299':
                return 'rs3_release'
            case '17134':
                return 'rs4_release'
            case '17763':
                return 'rs5_release'
            case '17784':
                return 'rs5_release_svc_hci'
            case '18362':
                return '19h1_release'
            case '18363':
                return '19h2_release'
            case '19041' | '19042' | '19043' | '19044':
                return 'vb_release'
            case '20348':
                return 'fe_release'
            case '22000':
                return 'co_release'
            case _:
                return 'rs_prerelease'

    def compose(self, build: str, sku_id: int, channel: str, defer_updates: int, *, attr_data_ver: int = 168, branch_readiness_level: str = 'CB', du_internal: int = 0, du_scan: int = 1, secure_boot_capable: int = 1, telemetry_level: int = 3, tpm_version: int = 2) -> str:
        match channel:
            case 'Retail':
                flight_ring = 'Retail'
                is_flighting_enabled = 0
            case _:
                flight_ring = 'External'
                is_flighting_enabled = 1

        device_attributes = {
            'AttrDataVer': attr_data_ver,
            'BranchReadinessLevel': branch_readiness_level,
            'DeferFeatureUpdatePeriodInDays': defer_updates,
            'DeferQualityUpdatePeriodInDays': defer_updates,
            'DUInternal': du_internal,
            'DUScan': du_scan,
            'FlightingBranchName': channel,
            'FlightRing': flight_ring,
            'IsFlightingEnabled': is_flighting_enabled,
            'MediaVersion': build,
            'OSSkuId': sku_id,
            'OSVersion': build,
            'SecureBootCapable': secure_boot_capable,
            'TelemetryLevel': telemetry_level,
            'TPMVersion': tpm_version,
        }

        return 'E:' + '&'.join(f'{k}={v}' for k, v in device_attributes.items())


class ProductsComposer:

    def __init__(self) -> None:
        pass

    def _get_branch_from_build(self, build: str) -> str:
        build = re.match(r'^(\d+)\.(\d+)\.(\d+)\.(\d+)$', build).group(3)

        match build:
            case '15063':
                return 'rs2_release'
            case '16299':
                return 'rs3_release'
            case '17134':
                return 'rs4_release'
            case '17763':
                return 'rs5_release'
            case '17784':
                return 'rs5_release_svc_hci'
            case '18362':
                return '19h1_release'
            case '18363':
                return '19h2_release'
            case '19041' | '19042' | '19043' | '19044':
                return 'vb_release'
            case '20348':
                return 'fe_release'
            case '22000':
                return 'co_release'
            case _:
                return 'rs_prerelease'

    def _get_product_name_from_sku_id(self, sku_id: int) -> str:
        match sku_id:
            case 4 | 27 | 48 | 49 | 72 | 84 | 98 | 100 | 101 | 119 | 121 | 122 | 125 | 126 | 129 | 130 | 161 | 162 | 163 | 164:
                return 'Client.OS.rs2'
            case 7 | 8 | 12 | 13 | 42 | 79 | 80 | 120 | 145 | 146 | 147 | 158 | 159 | 160 | 406 | 407 | 408:
                return 'Server.OS'

    def compose(self, build: str, sku_id: int, arches: list) -> str:
        return ';'.join(f'PN={self._get_product_name_from_sku_id(sku_id)}.{arch}&Branch={self._get_branch_from_build(build)}&PrimaryOSProduct=1&Repairable=1&V={build}' for arch in arches)
